# -*- coding: utf-8 -*-

# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Provides a Python library to extend the Ansys SCADE IDE GUI."""

from pathlib import Path
import sys

try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:
    import importlib_metadata

try:
    __version__ = importlib_metadata.version(__name__.replace('.', '-'))
except importlib_metadata.PackageNotFoundError:
    # happens with pre-commit, the package is not installed in the virtual environment
    __version__ = '<unknown>'


def get_srg_name() -> str:
    """
    Return the name of the registration file for Ansys SCADE IDE.

    It addresses SCADE 2024 R1 and prior releases.
    SCADE 2024 R2 and later use the package's
    ``ansys.scade.registry`` entry point.
    """
    # registrations depending on Python interpreter
    python_version = str(sys.version_info.major) + str(sys.version_info.minor)
    suffix = '23r1' if python_version == '37' else '24r1'
    return f'guitools{suffix}.srg'


def srg() -> str:
    r"""
    Return the path of the SCADE Studio registry file.

    This function implements the entry point "ansys.scade.registry/srg"
    introduced in SCADE 2024 R2. It avoids creating an explicit srg file
    in ``%APPDATA%\Scade\Customize`` when the package is installed.
    """
    # the package's srg file is located in the same directory
    return str(Path(__file__).parent / 'guitools.srg')



# -*- coding: utf-8 -*-

# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Provides an extension for the Page classes."""

from abc import abstractmethod
from typing import Any, Callable, List, Optional, Tuple, Union

from scade.model.project.stdproject import Configuration, Project
from scade.tool.suite.gui.properties import Page as PropertyPage
from scade.tool.suite.gui.settings import Page as SettingsPage

from ansys.scade.guitools.control import (
    FSM,
    CheckButton,
    ComboBox,
    Edit,
    FileSelector,
    GroupRadioBox,
    ObjectComboBox,
    RadioBox,
    StaticComboBox,
    StaticEdit,
    StaticObjectComboBox,
    StaticRadioBox,
)
import ansys.scade.guitools.csts as c
from ansys.scade.guitools.interfaces import (
    IGuiHostClient,
    IPropertiesDataExchange,
    ISettingsDataExchange,
)

# width of fields, second column: unused since the controls are sized automatically
_WF = 100

Page = Union[PropertyPage, SettingsPage]

Getter = Callable[[], Any]
"""Signature for getting control's value."""
Setter = Callable[[Any], None]
"""Signature for setting control's value."""


class ContainerPage:
    """
    Base class for property or settings pages.

    It maintains a list of controls for automatic layout,
    or to show or hide an entire page.

    The controls are automatically added on two columns: labels and edits.

    Parameters
    ----------
    page : Page | None
        Owner page for controls.

        If this parameter is not known at initialization, make sure
        the attribute ``self.page`` is defined before creating any control.

    label_width : int
        Width of the first column.
    """

    def __init__(self, page: Optional[Page], label_width: int):
        self.page = page
        self.label_width = label_width
        self.controls = []

    def add_edit(self, y: int, **kwargs) -> Edit:
        """Add a :class:`Edit <ansys.scade.guitools.control.Edit>` control to the page."""
        x = kwargs.pop('x', c.LEFT_MARGIN)
        edit = Edit(self.page, x, y, _WF, **kwargs)
        self.controls.append(edit)
        return edit

    def add_static_edit(self, y: int, text: str, **kwargs) -> StaticEdit:
        """Add a :class:`StaticEdit <ansys.scade.guitools.control.StaticEdit>` control to the page."""
        wl = kwargs.pop('wl', self.label_width)
        x = kwargs.pop('x', c.LEFT_MARGIN)
        edit = StaticEdit(self.page, text, wl, x, y, _WF, **kwargs)
        self.controls.append(edit)
        return edit

    def add_file_selector(
        self, y: int, text: str, extension: str, dir: str, filter: str, mode: FSM, **kwargs
    ) -> FileSelector:
        """Add a :class:`FileSelector <ansys.scade.guitools.control.FileSelector>` to the page."""
        wl = kwargs.pop('wl', self.label_width)
        x = kwargs.pop('x', c.LEFT_MARGIN)
        file = FileSelector(
            self.page,
            text,
            extension,
            dir,
            filter,
            mode,
            wl,
            x,
            y,
            _WF,
            **kwargs,
        )
        self.controls.append(file)
        return file

    def add_check_button(self, y: int, text: str, **kwargs) -> CheckButton:
        """Add a :class:`CheckButton <ansys.scade.guitools.control.CheckButton>` control to the page."""
        x = kwargs.pop('x', c.LEFT_MARGIN)
        w = kwargs.pop('w', self.label_width)
        cb = CheckButton(self.page, text, x, y, w, **kwargs)
        self.controls.append(cb)
        return cb

    def add_combo_box(self, y: int, text: str, **kwargs) -> ComboBox:
        """Add a :class:`ComboBox <ansys.scade.guitools.control.ComboBox>` control to the page."""
        x = kwargs.pop('x', c.LEFT_MARGIN)
        cb = ComboBox(self.page, x, y, _WF, **kwargs)
        self.controls.append(cb)
        return cb

    def add_object_combo_box(self, y: int, text: str, **kwargs) -> ObjectComboBox:
        """Add a :class:`ObjectComboBox <ansys.scade.guitools.control.ObjectComboBox>` control to the page."""
        x = kwargs.pop('x', c.LEFT_MARGIN)
        cb = ObjectComboBox(self.page, x, y, _WF, **kwargs)
        self.controls.append(cb)
        return cb

    def add_static_combo_box(self, y: int, text: str, **kwargs) -> StaticComboBox:
        """Add a :class:`StaticComboBox <ansys.scade.guitools.control.StaticComboBox>` control to the page."""
        wl = kwargs.pop('wl', self.label_width)
        x = kwargs.pop('x', c.LEFT_MARGIN)
        cb = StaticComboBox(self.page, text, wl, x, y, _WF, **kwargs)
        self.controls.append(cb)
        return cb

    def add_static_object_combo_box(self, y: int, text: str, **kwargs) -> StaticObjectComboBox:
        """Add a :class:`StaticObjectComboBox <ansys.scade.guitools.control.StaticObjectComboBox>` control to the page."""
        wl = kwargs.pop('wl', self.label_width)
        x = kwargs.pop('x', c.LEFT_MARGIN)
        cb = StaticObjectComboBox(self.page, text, wl, x, y, _WF, **kwargs)
        self.controls.append(cb)
        return cb

    def add_radio_box(self, y: int, buttons: List[Tuple[Any, str]], **kwargs) -> RadioBox:
        """Add a :class:`RadioBox <ansys.scade.guitools.control.RadioBox>` control to the page."""
        x = kwargs.pop('x', c.LEFT_MARGIN)
        rb = RadioBox(self.page, buttons, x, y, _WF, **kwargs)
        self.controls.append(rb)
        return rb

    def add_group_radio_box(
        self, y: int, text: str, buttons: List[Tuple[Any, str]], **kwargs
    ) -> GroupRadioBox:
        """Add a :class:`GroupRadioBox <ansys.scade.guitools.control.GroupRadioBox>` control to the page."""
        x = kwargs.pop('x', c.LEFT_MARGIN)
        grb = GroupRadioBox(self.page, text, buttons, x, y, _WF, **kwargs)
        self.controls.append(grb)
        return grb

    def add_static_radio_box(
        self, y: int, text: str, buttons: List[Tuple[Any, str]], **kwargs
    ) -> StaticRadioBox:
        """Add a :class:`StaticRadioBox <ansys.scade.guitools.control.StaticRadioBox>` control to the page."""
        wl = kwargs.pop('wl', self.label_width)
        x = kwargs.pop('x', c.LEFT_MARGIN)
        srb = StaticRadioBox(self.page, text, wl, buttons, x, y, _WF, **kwargs)
        self.controls.append(srb)
        return srb

    def add_control(self, control):
        """Add an existing control to the page's list of controls."""
        self.controls.append(control)

    def layout_controls(self):
        """Declare the contained control's constraints."""
        for control in self.controls:
            try:
                control.on_layout()
            except AttributeError:
                # ignore exception for controls not defining the function
                pass

    def show_controls(self, show: bool):
        """Show or hide the contained controls."""
        for control in self.controls:
            control.set_visible(show)


class SettingsPageEx(SettingsPage, ContainerPage):
    """
    Provides a base class for settings pages.

    It introduces a new abstract method that must be implemented
    for building the controls.
    """

    def __init__(self, label_width: int, *args, **kwargs):
        super().__init__(*args, **kwargs)
        super(SettingsPage, self).__init__(self, label_width)
        self.ddx = None  # type: Optional[ISettingsDataExchange]

    def on_build(self):
        """Build the settings page."""
        # reset the list of controls
        self.controls = []
        self.ddx = self.on_build_ex()

    def on_layout(self):
        """Specify how controls are moved or resized."""
        self.layout_controls()

    def on_display(self, project: Project, configuration: Configuration):
        """Update the page with the properties read from the project."""
        if self.ddx:
            self.ddx.model_to_page(project, configuration)

    def on_validate(self, project: Project, configuration: Configuration):
        """Update the project with the properties read from the page."""
        if self.ddx:
            self.ddx.page_to_model(project, configuration)

    @abstractmethod
    def on_build_ex(self) -> Optional[ISettingsDataExchange]:
        """
        Build the controls and return an optional object for managing the persistence.

        Returns
        -------
        Optional[ISettingsDataExchange]
            Object for exchanging data between the project and the controls.
        """
        raise NotImplementedError


class PropertyPageEx(PropertyPage, ContainerPage):
    """
    Provides a base class for property pages.

    This class also provides means to manage the sizing
    of most common controls and their serialization.

    It introduces a new abstract method that must be implemented
    for building the controls.

    Parameters
    ----------
    label_width : int
        Width of the labels, displayed before the controls on the same line.
    """

    def __init__(self, label_width: int, *args, **kwargs):
        super().__init__(*args, **kwargs)
        super(PropertyPage, self).__init__(self, label_width)
        self.ddx = None  # type: Optional[IPropertiesDataExchange]
        self.models = []  # type: List[Any]

    def on_build(self):
        """Build the properties page."""
        # reset the list of controls
        self.controls = []
        self.ddx = self.on_build_ex()

    def on_layout(self):
        """Specify how controls are moved or resized."""
        self.layout_controls()

    def on_context(self, models: List[Any]):
        """
        Declare the models the page should consider.

        Parameters
        ----------
        models : List[Any]
            List of selected objects in the IDE.
        """
        self.models = models

    def on_display(self):
        """Update the page with the properties read from the models."""
        if self.ddx:
            for model in self.models:
                self.ddx.model_to_page(model)

    def on_validate(self):
        """Update the models with the properties read from the page."""
        if self.ddx:
            for model in self.models:
                self.ddx.page_to_model(model)

    @abstractmethod
    def on_build_ex(self) -> Optional[IPropertiesDataExchange]:
        """
        Build the controls and return an optional object for managing the persistence.

        Returns
        -------
        Optional[IPropertiesDataExchange]
            Object for exchanging data between the models and the controls.
        """
        raise NotImplementedError


class GuiHostClientPage(IGuiHostClient, ContainerPage):
    """Default implementation for GuiHost pages."""

    def __init__(self, *args, **kwargs):
        super(IGuiHostClient, self).__init__(page=None, *args, **kwargs)
        self.ddx = None  # type: Optional[IPropertiesDataExchange]

    def get_selected_models(self, models: List[Any]) -> List[Any]:
        """
        Return a new list of models from the selection.

        For example, replaces selected graphical elements by their
        associated semantic ones.

        Parameters
        ----------
        models : List[Any]
            List of selected objects in the IDE.

        Returns
        -------
        List[Any]
            List of objects to consider.
        """
        return models

    def is_available(self, models: List[Any]) -> bool:
        """
        Return whether the page is available for the current selection.

        Parameters
        ----------
        models : List[Any]
            List of selected objects in the IDE.
        """
        return len(self.get_selected_models(models)) > 0

    def set_models(self, models: List[Any]):
        """
        Declare the models the page should consider.

        Parameters
        ----------
        models : List[Any]
            List of selected objects in the IDE.
        """
        self.models = self.get_selected_models(models)

    def show(self, show: bool):
        """
        Show or hide the page.

        This consists in showing or hiding the contained controls.

        Parameters
        ----------
        show : bool
            Whether the page should be shown or hidden.
        """
        self.show_controls(show)

    def on_layout(self):
        """Declare the contained control's constraints."""
        self.layout_controls()

    def on_display(self):
        """Update the page with the properties read from the models."""
        if self.ddx:
            for model in self.models:
                self.ddx.model_to_page(model)

    def on_validate(self):
        """Update the models with the properties read from the page."""
        if self.ddx:
            for model in self.models:
                self.ddx.page_to_model(model)

    def on_build(self, page: PropertyPage, y: int):
        """Build the property page."""
        # update page attribute that wasn't known at initialization
        self.page = page
        # reset the list of controls
        self.controls = []
        # build the controls
        self.ddx = self.on_build_ex(y)

    def on_close(self):
        """Perform any cleaning before the page is closed."""
        pass

    @abstractmethod
    def on_build_ex(self, y: int) -> Optional[IPropertiesDataExchange]:
        """
        Build the controls and return an optional object for managing the persistence.

        Parameters
        ----------
        y : int
            Start vertical position.

        Returns
        -------
        Optional[IPropertiesDataExchange]
            Object for exchanging data between the models and the controls.
        """
        raise NotImplementedError



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
Registers the Ansys SCADE GUI Tools extension registry files (SRG).

Refer to the :ref:`installation <getting_started_install_user>`
steps for more information.

It addresses SCADE 2024 R1 and prior releases.
SCADE 2024 R2 and later use the package's
``ansys.scade.registry`` entry point.
"""

import os
from pathlib import Path
import sys
from typing import Tuple

from ansys.scade.guitools import get_srg_name

# APPDATA must be defined
_APPDATA = os.environ['APPDATA']


def _register_srg_file(srg: Path, install: Path):
    # copy the srg file to Customize and patch it with the installation directory.
    text = srg.read_text(encoding='utf-8')
    text = text.replace('%TARGETDIR%', install.as_posix())
    dst = Path(_APPDATA, 'SCADE', 'Customize', srg.name)
    dst.parent.mkdir(parents=True, exist_ok=True)
    dst.write_text(text, encoding='utf-8')


def register() -> Tuple[int, str]:
    """Implement the ``ansys.scade.registry/register`` entry point."""
    script_dir = Path(__file__).parent
    _register_srg_file(script_dir / get_srg_name(), script_dir)
    return (0, '')


def main() -> int:
    """Implement the ``ansys.scade.guitools.register`` packages's project script."""
    code, message = register()
    if message:
        print(message, file=sys.stderr if code else sys.stdout)
    return code


if __name__ == '__main__':
    code = main()
    sys.exit(code)



# -*- coding: utf-8 -*-

# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Provides access to GUI server for property pages."""

import traceback
from typing import Any, Dict, List

import scade

from ansys.scade.apitools.info import get_scade_version
from ansys.scade.guitools import __version__, importlib_metadata
from ansys.scade.guitools.control import ComboBox
import ansys.scade.guitools.csts as c
from ansys.scade.guitools.interfaces import IGuiHostClient
from ansys.scade.guitools.page import PropertyPageEx

_pages = {}


class ProxyPageClient:
    """
    Maintains a reference to a client page and the name of its category.

    Parameters
    ----------
    category : str
        Name of the page.

    client : IGuiHostClient
        Instance of the hosted page.
    """

    def __init__(self, category: str, client: IGuiHostClient):
        """Initialize the proxy with a category and a client."""
        self.category = category
        self.client = client


class HostPage(PropertyPageEx):
    """
    Defines a property page for hosting client pages.

    Parameters
    ----------
    name : str
        Name of the property Page.

    optional : bool
        Whether the combo box for selecting clients is hidden
        when there is only one client.

    args : Any
        Additional parameters for the property page.

    kwargs : Any
        Additional parameters for the property page.
    """

    def __init__(self, name: str, optional: bool, *args, **kwargs):
        """Initialize the host page."""
        super().__init__(50, name=name, *args, **kwargs)
        self.optional = optional

        # registered clients
        self.proxies: List[ProxyPageClient] = []
        # active clients for the current selection, indexed by category
        self.active_clients: Dict[str, ProxyPageClient] = {}
        # active categories (sorted)
        self.categories: List[str] = []
        # current selected category
        self.category = ''
        # controls
        self.cb_clients = None

    def add_client(self, proxy: ProxyPageClient):
        """
        Add a client to the page.

        Parameters
        ----------
        proxy : ProxyPageClient
            Client page proxy.
        """
        self.proxies.append(proxy)

    def is_available(self, models: List[Any]) -> bool:
        """
        Return whether the page is available for the current selection.

        The page is available for the selected models
        if at least one of its clients is available

        Parameters
        ----------
        models : List[Any]
            List of selected objects in the IDE.
        """
        self.active_clients = {}
        self.categories = []
        for proxy in sorted(self.proxies, key=lambda p: p.category):
            if proxy.client.is_available(models):
                self.active_clients[proxy.category] = proxy
                self.categories.append(proxy.category)
        return len(self.categories) > 0

    def on_context(self, models: List[Any]):
        """
        Declare the models the page should consider.

        Parameters
        ----------
        models : List[Any]
            List of selected objects in the IDE.
        """
        if not models:
            # deactivate the clients
            for proxy in self.active_clients.values():
                proxy.client.set_models(models)
            self.active_clients = {}
        else:
            # called between is_available and on_display,
            # seems redundant with is_available
            for proxy in self.active_clients.values():
                if not proxy.client.is_available(models):
                    if not models:
                        continue
                # assert proxy.client.is_available(models)
                proxy.client.set_models(models)

    def on_build(self):
        """Build the property page and its clients."""
        # alignment for the first line
        y = c.TOP_MARGIN
        self.cb_clients = self.add_static_combo_box(
            y, '&Tool:', style=['dropdownlist'], on_change_selection=self.on_sel_change
        )
        self.cb_clients.set_items(self.categories)
        if len(self.active_clients) > 1:
            y += c.DY
        else:
            if self.optional:
                self.cb_clients.set_visible(False)
            else:
                self.cb_clients.set_enable(False)
                y += c.DY
        for proxy in self.active_clients.values():
            proxy.client.on_build(self, y)

    def on_display(self):
        """
        Display the property page and its clients.

        Activate the last active client when possible.
        """
        assert self.cb_clients is not None  # nosec B101  # addresses linter
        selected_proxy = self.active_clients.get(self.category)
        if not selected_proxy:
            self.category = self.categories[0]
            selected_proxy = self.active_clients.get(self.category)
        self.cb_clients.set_selection(self.category)
        for proxy in self.active_clients.values():
            proxy.client.on_display()
            proxy.client.show(proxy == selected_proxy)

    def on_validate(self):
        """Validate the property page's clients."""
        for proxy in self.active_clients.values():
            proxy.client.on_validate()

    def on_close(self):
        """Perform any cleaning before the page is closed."""
        for proxy in self.active_clients.values():
            proxy.client.on_close()

    def on_layout(self):
        """Declare the contained control's constraints."""
        assert self.cb_clients is not None  # nosec B101  # addresses linter
        self.cb_clients.on_layout()
        for proxy in self.active_clients.values():
            proxy.client.on_layout()

    def on_sel_change(self, combobox: ComboBox, index: int):
        """
        Display the selected client.

        Parameters
        ----------
        combobox : ComboBox
            Control initiating the notification. Unused.

        index : int
            Index of the selected element.
        """
        new_category = self.categories[index]
        if new_category != self.category:
            self.active_clients[self.category].client.show(False)
            self.category = new_category
            self.active_clients[self.category].client.show(True)


def main():
    """Create the server property pages from the installed clients."""
    global _pages

    # get the current version and convert it to the format of srg files
    std_version = get_scade_version() * 100

    # load the page descriptions from the registered entry points
    group = 'ansys.scade.guihost'
    descriptions = sum(
        [_.load()() for _ in importlib_metadata.entry_points(group=group) if _.name == 'pages'], []
    )

    for description in descriptions:
        version = description.get('version', 0)
        expire = description.get('expire', 9999999)
        if version > std_version or expire <= std_version:
            continue
        # deprecated
        if not description.get('activate', True):
            continue
        if not description.get('active', True):
            continue
        tab = description['page']
        category = description['category']
        class_ = description['class']
        # do not display the categories when optional and only one category
        optional = description.get('optional', False)

        # get the host page for the category
        page = _pages.get(tab)
        if not page:
            page = HostPage(tab, optional)
            _pages[tab] = page
        # create the instance of the hosted page
        cls = class_()
        page.add_client(ProxyPageClient(category, cls))


# scade is a CPython module defined dynamically
try:
    main()
except BaseException as e:
    scade.tabput('LOG', f'{e}\n')  # type: ignore
    scade.tabput('LOG', f'{traceback.format_exc()}\n')  # type: ignore
else:
    scade.tabput('LOG', f'Loading GUI Host {__version__}\n')  # type: ignore



# -*- coding: utf-8 -*-

# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
Provides a SCADE IDE Command base class.

This module provides Command base class that stubs
``scade.tool.suite.gui.commands.Command`` when not available.

This allows to create unit tests for commands.
"""

from ansys.scade.guitools.ide import Ide

try:
    from scade.tool.suite.gui.commands import Command as _Command  # type: ignore
except ImportError:
    import scade

    scade.output('IDE stub activated\n')  # type: ignore

    class _Command:
        """Stub for ``scade.tool.suite.gui.commands.Command``."""

        def __init__(
            self,
            name: str,
            status_message: str,
            tooltip_message: str,
            image_file: str = '',
        ):
            self.name = name
            self.status_message = status_message
            self.tooltip_message = tooltip_message
            self.image_file = image_file

        def on_enable(self) -> bool:
            """Return whether the command can be activated, ``True`` by default."""
            return True


class Command(_Command):
    """
    Base class for commands.

    Parameters
    ----------
    ide : Ide
        SCADE IDE abstraction.
    """

    def __init__(self, ide: Ide, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.ide = ide



# -*- coding: utf-8 -*-

# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Provides an extension for the Dialog class."""

from abc import abstractmethod
from enum import Enum

from scade.tool.suite.gui.dialogs import Dialog
from scade.tool.suite.gui.widgets import Button

import ansys.scade.guitools.csts as c


class DS(Enum):
    """Style for dialog box validation buttons."""

    NONE, CLOSE, OK_CANCEL, RETRY_CANCEL, YES_NO, YES_NO_CANCEL = range(6)


class DialogBox(Dialog):
    """
    Defines a dialog box with optional default button management.

    Parameters
    ----------
    title: str
        Title of the dialog box.

    width: int
        Width of the client area if ``nc`` is ``False``, otherwise
        overall width of the dialog box.

    height: int
        Height of the client area if ``nc`` is ``False``, otherwise
        overall width of the dialog box.

    nc: bool
        Whether the provided dimensions are for the overall dialog box
        or for its client area, default ``False``.

    kwargs: Any
        Other parameters of ``scade.tool.suite.gui.dialog.Dialog``.
    """

    def __init__(
        self, title: str, width: int, height: int, nc: bool = False, style: DS = DS.NONE, **kwargs
    ):
        """Initialize the dialog box."""
        if not nc:
            # increase the bounding box with NC margins
            width += c.NC_RIGHT + c.NC_LEFT
            height += c.NC_TOP + c.NC_BOTTOM
        super().__init__(title, width, height, **kwargs)

        # store instantiation parameters
        self._nc = nc
        self._style = style
        # buttons, depending on style
        self._buttons = []

    @property
    def right(self) -> int:
        """
        Return the right-most position of the dialog box.

        This corresponds to the width of its client area.
        """
        return self.width - (c.NC_RIGHT + c.NC_LEFT)

    @property
    def bottom(self) -> int:
        """
        Return the bottom-most position of the dialog box.

        This corresponds to the height of its client area
        if there are no buttons, otherwise the vertical position
        of the bottom buttons.
        """
        bottom = self.height - (c.NC_TOP + c.NC_BOTTOM)
        if self._style != DS.NONE:
            bottom -= c.BUTTON_HEIGHT + c.BOTTOM_MARGIN
        return bottom

    def on_build(self):
        """Build the dialog with the specified dialog validation buttons."""
        # build the controls
        self.on_build_ex()
        # add the validation buttons
        descriptions = [
            ('Close', self.on_click_close, (DS.CLOSE,)),
            ('OK', self.on_click_ok, (DS.OK_CANCEL,)),
            ('Retry', self.on_click_retry, (DS.RETRY_CANCEL,)),
            ('&Yes', self.on_click_yes, (DS.YES_NO, DS.YES_NO_CANCEL)),
            ('&No', self.on_click_no, (DS.YES_NO, DS.YES_NO_CANCEL)),
            ('Cancel', self.on_click_cancel, (DS.OK_CANCEL, DS.RETRY_CANCEL, DS.YES_NO_CANCEL)),
        ]
        selection = [_ for _ in descriptions if self._style in _[-1]]
        self._buttons = []
        # separation between two buttons
        separator = c.RIGHT_MARGIN
        if selection:
            count = len(selection)
            # coordinates relative to the client area
            # dialog's dimensions include the non-client area
            x = self.right - count * (c.BUTTON_WIDTH + separator)
            y = self.bottom
            for label, callback, _ in selection:
                button = Button(self, label, x, y, c.BUTTON_WIDTH, c.BUTTON_HEIGHT, callback)
                self._buttons.append(button)
                x += c.BUTTON_WIDTH + separator

    def on_click_close(self, *args):
        """Close the dialog when Close is pressed."""
        self.close()

    def on_click_yes(self, *args):
        """Close the dialog when Yes is pressed."""
        self.close()

    def on_click_no(self, *args):
        """Close the dialog when No is pressed."""
        self.close()

    def on_click_ok(self, *args):
        """Close the dialog when OK is pressed."""
        self.close()

    def on_click_cancel(self, *args):
        """Close the dialog when Cancel is pressed."""
        self.close()

    def on_click_retry(self, *args):
        """Close the dialog when Retry is pressed."""
        self.close()

    @abstractmethod
    def on_build_ex(self):
        """Build the controls."""
        raise NotImplementedError



# -*- coding: utf-8 -*-

# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
Provides default metrics for designing dialog boxes and controls.

These metrics may depend on your version of Windows or selected theme.
"""

# NC area (Non client area) of dialogs
# If the bounding box of a dialog is 200x100, its client area is 184x61
H_TITLE = 28
"""Default height of the title area of a dialog box."""
NC_TOP = H_TITLE + 3
"""Non client area top margin of a dialog box."""
NC_LEFT = 8
"""Non client area left margin of a dialog box."""
NC_RIGHT = 8
"""Non client area right margin of a dialog box."""
NC_BOTTOM = 8
"""Non client area bottom margin of a dialog box."""

DY = 30
"""Default vertical offset between 2 lines."""

RIGHT_MARGIN = 6
"""Right margin of a dialog box or page."""
LEFT_MARGIN = 6
"""Left margin of a dialog box or page."""
TOP_MARGIN = 7
"""Top margin of a dialog box or page."""
BOTTOM_MARGIN = 7
"""Bottom margin of a dialog box or page."""

BUTTON_HEIGHT = 23
"""Default height of a push button control."""
BUTTON_WIDTH = 75
"""Default width of a push button control."""

STATIC_HEIGHT = 16
"""Default height of a static control."""

EDIT_HEIGHT = 20
"""Default height of an edit control."""

# height adjusted to the edit control usually associated to this button
DOTS_HEIGHT = BUTTON_HEIGHT - 1
"""Default height of the ``...`` push button control."""
DOTS_WIDTH = 30
"""Default width of a ``...`` push button control."""

CHECK_BUTTON_HEIGHT = 20
"""Default height of a check button control."""

COMBO_BOX_HEIGHT = 130
"""Default height of a combo box control."""

RADIO_BUTTON_HEIGHT = 20
"""Default height of a radio button control."""

GROUP_RADIO_BOX_HEIGHT = RADIO_BUTTON_HEIGHT + STATIC_HEIGHT + BOTTOM_MARGIN
"""Default height of a group radio box control."""

RADIO_BOX_DY = GROUP_RADIO_BOX_HEIGHT + (DY - EDIT_HEIGHT)
"""Default vertical offset for a group radio box control."""



# -*- coding: utf-8 -*-

# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Provides extensions for the persistence of settings and properties."""

from typing import (
    Any,
    Callable,
    List,  # noqa: F401  # used in a typing annotation
    Optional,
    Tuple,
)

from scade.model.project.stdproject import Configuration, Project, ProjectEntity
import scade.model.suite as suite
from scade.tool.suite.gui.widgets import CheckBox, EditBox, ListBox, RadioButton, Widget

from ansys.scade.apitools.prop import get_pragma_json, set_pragma_json
from ansys.scade.guitools.control import ComboBox, ObjectComboBox, RadioBox
from ansys.scade.guitools.interfaces import IPropertiesDataExchange, ISettingsDataExchange

Getter = Callable[[], Any]
"""Signature for getting control value."""
Setter = Callable[[Any], None]
"""Signature for setting control value."""


class DataExchange:
    """Base class for accessing controls data."""

    def __init__(self):
        """Initialize the data exchange instance."""
        # get, set, name, default, empty
        self.properties = []  # type: List[Tuple[Getter, Setter, str, Any, Any]]

    def get_control_accessors(self, control: Widget) -> Tuple[Optional[Getter], Optional[Setter]]:
        """
        Return the get and set accessors for a control, or None if not applicable.

        Override this method in a derived class to define custom
        implementation for not supported controls.

        Parameters
        ----------
        control : Widget
            Input control.

        Returns
        -------
        tuple[Getter, Setter]
        """
        if isinstance(control, EditBox):
            return (control.get_name, control.set_name)
        elif isinstance(control, CheckBox):
            return (control.get_check, control.set_check)
        elif isinstance(control, RadioButton):
            return (control.get_check, control.set_check)
        elif isinstance(control, ComboBox):
            return (control.get_name, control.set_name)
        elif isinstance(control, ListBox):
            return (control.get_selection, control.set_selection)
        elif isinstance(control, ObjectComboBox):
            return (control.get_selected_name, control.select_name)
        elif isinstance(control, RadioBox):
            return (control.get_value, control.set_value)
        # TODO(Jean Henry): keep or remove logic
        # https://github.com/ansys/scade-guitools/issues/26
        # elif isinstance(control, ObjectListBox):
        #     return (control.get_selected_names, control.select_names)

        return None, None

    def ddx_control(self, control: Widget, name: str, default: Any, empty: Any = None):
        """
        Declare a property for automatic serialization.

        When the page is displayed, the control is updated with the value read from the model.
        When the page is validated, the model is updated with the value read from the control.

        Parameters
        ----------
        control : Widget
            Control associated to the property.

        name :
            Name of the property.

        default : Any
            Default value of the property.

        empty : Any | None
            Value to display when it is empty, default ``default``.

        Examples
        --------

        .. code-block::

            # on_build_ex method of a page
            self.tp = ToolPropDataExchange('MY_TOOL')
            ...
            edit = self.add_edit(y)
            self.tp.ddx_control(edit, name='MY_PROP', default='')
            y += csts.DY
            cb = self.add_check_button(y, 'Option')
            self.tp.ddx_control(cb, name='MY_OPTION', default=False)
            y += csts.DY
        """
        pfnget, pfnset = self.get_control_accessors(control)
        if pfnget:
            assert pfnset is not None  # nosec B101  # addresses linter
            if empty is None:
                empty = default
            self.properties.append((pfnget, pfnset, name, default, empty))


class ToolPropDataExchange(DataExchange):
    """
    Means to serialize values as project *tool properties*.

    A *tool property* is a project property which name has the following syntax:
    ``@<TOOL>:<NAME>``, where ``<TOOL>`` is a name for discriminating homonymous
    properties and ``<NAME>`` is the name of the property.

    For example: ``@GENERATOR:TARGET_DIR``

    Parameters
    ----------
    tool : str
        Name of the tool to build the project property's name: first token after ``@``.
    """

    def __init__(self, tool: str):
        """Initialize the tool property data exchange instance."""
        super().__init__()
        self.tool = tool

    def display(self, project: Project, configuration: Optional[Configuration]):
        """Update the page with the properties read from the model."""
        for _, pfnset, name, default, empty in self.properties:
            if isinstance(default, list):
                value = project.get_tool_prop_def(self.tool, name, default, configuration)
            elif isinstance(default, bool):
                value = project.get_bool_tool_prop_def(self.tool, name, default, configuration)
            else:
                # assume a scalar value
                value = project.get_scalar_tool_prop_def(self.tool, name, default, configuration)
            if not value and empty:
                value = empty
            pfnset(value)

    def validate(self, project: Project, configuration: Optional[Configuration]):
        """Update the model with the properties read from the page."""
        for pfnget, _, name, default, empty in self.properties:
            value = pfnget()
            if value == empty:
                value = default
            if isinstance(default, list):
                project.set_tool_prop_def(self.tool, name, value, default, configuration)
            elif isinstance(value, bool):
                project.set_bool_tool_prop_def(self.tool, name, value, default, configuration)
            else:
                # assume a scalar value
                project.set_scalar_tool_prop_def(self.tool, name, value, default, configuration)


class SettingsDataExchange(ISettingsDataExchange, ToolPropDataExchange):
    """Default implementation to manage the persistence of most usual controls in the project."""

    def __init__(self, tool: str):
        """Initialize the settings data exchange instance."""
        super().__init__(tool)
        # super(IPropertiesDataExchange, self).__init__(self, tool)

    def model_to_page(self, project: Project, configuration: Configuration):
        """Update the page with the properties read from the model."""
        self.display(project, configuration)

    def page_to_model(self, project: Project, configuration: Configuration):
        """Update the model with the properties read from the page."""
        self.validate(project, configuration)


class ProjectPropertiesDataExchange(IPropertiesDataExchange, ToolPropDataExchange):
    """Default implementation to manage the persistence of most usual controls in the project."""

    def __init__(self, tool: str):
        """Initialize the project properties data exchange instance."""
        super().__init__(tool)
        # super(IPropertiesDataExchange, self).__init__(self, tool)

    def model_to_page(self, model: ProjectEntity):
        """Update the page with the properties read from the model."""
        self.display(model, None)

    def page_to_model(self, model: ProjectEntity):
        """Update the model with the properties read from the page."""
        self.validate(model, None)


class PragmaDataExchange(DataExchange):
    """
    Means to serialize values as SCADE Suite pragmas.

    SCADE Suite pragmas are either text or XML. This class
    uses a textual pragma made of a json string.

    Parameters
    ----------
    id : str
        Identifier of the pragma.
    """

    def __init__(self, id: str):
        """Initialize the pragma data exchange instance."""
        super().__init__()
        self.id = id

    def display(self, object_: suite.Annotable):
        """Update the page with the properties read from the model."""
        data = get_pragma_json(object_, self.id)
        if data is None:
            # defensive programming, json syntax error error not expected
            # assert False
            return
        assert isinstance(data, dict)  # nosec B101  # addresses linter
        for _, pfnset, name, default, empty in self.properties:
            value = data.get(name, default)
            if not value and empty:
                value = empty
            pfnset(value)

    def validate(self, object_: suite.Annotable):
        """Update the model with the properties read from the page."""
        data = {}
        for pfnget, _, name, default, empty in self.properties:
            value = pfnget()
            if value == empty:
                value = default
            if value != default:
                data[name] = value
        set_pragma_json(object_, self.id, data)


class ScadePropertiesDataExchange(IPropertiesDataExchange, PragmaDataExchange):
    """Default implementation to manage the persistence of most usual controls in the model."""

    def __init__(self, id: str):
        """Initialize the SCADE properties data exchange instance."""
        super().__init__(id)
        # super(IPropertiesDataExchange, self).__init__(self, id)

    def model_to_page(self, object_: suite.Annotable):
        """Update the page with the properties read from the model element."""
        self.display(object_)

    def page_to_model(self, object_: suite.Annotable):
        """Update the model element with the properties read from the page."""
        self.validate(object_)



# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""
Enables remote debugging with ``debugpy``.

Refer to :ref:`Debug <contributing_debug>` for more information.
"""

import os
from pathlib import Path

import debugpy

os.environ['PYDEVD_LOAD_NATIVE_LIB'] = '0'
os.environ['PYDEVD_USE_CYTHON'] = '0'

# get the python interpreter, can't use sys.executable which is VCS.EXE
_os_module_path = Path(os.__file__).resolve().parent
_python_exe = _os_module_path / '..' / 'python.exe'

debugpy.configure(python=str(_python_exe))


def attach_to_debugger(port: int = 5678):
    """Start the debug server and wait for a client to connect."""
    debugpy.listen(port)
    debugpy.wait_for_client()



# -*- coding: utf-8 -*-

# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""``Ide`` interface implementation."""

# this module uses `type: ignore` pragmas for ``scade`` functions: they
# are defined dynamically when the interpreter is created: typing analysis
# is not possible

# from collections.abc import Callable
from typing import Any, Callable, List, Optional, Tuple, Union

import scade
from scade.model import suite
from scade.model.project.stdproject import Configuration, Project, get_roots as get_projects
from scade.model.suite import get_roots as get_sessions

from ansys.scade.apitools import print
from ansys.scade.guitools.ide import Ide


class Studio(Ide):
    """Implementation for SCADE IDE."""

    # GUI-related Python commands

    def activate(self, object_: Any):
        """Redirect the call to SCADE IDE."""
        scade.activate(object_)  # type: ignore

    def activate_browser(self, tab_name: str):
        """Redirect the call to SCADE IDE."""
        scade.activate_browser(tab_name)  # type: ignore

    def activate_project(self, project_name: str):
        """Redirect the call to SCADE IDE."""
        scade.activate_project(project_name)  # type: ignore

    def activate_tab(self, tab: str):
        """Redirect the call to SCADE IDE."""
        scade.activate_tab(tab)  # type: ignore

    def browser_report(
        self,
        child_object: Any,
        parent_object: Any = None,
        expanded: bool = False,
        user_data: Any = None,
        name: str = '',
        icon_file: str = '',
    ):
        """Redirect the call to SCADE IDE."""
        # issue with 'icon_file' default value (2025 R2)
        if icon_file:
            scade.browser_report(child_object, parent_object, expanded, user_data, name, icon_file)  # type: ignore
        else:
            scade.browser_report(child_object, parent_object, expanded, user_data, name)  # type: ignore

    def clear_tab(self, tab: str):
        """Redirect the call to SCADE IDE."""
        scade.clear_tab(tab)  # type: ignore

    def command(
        self, extension_id: str, command_id: int, command: str = 'activate'
    ) -> Optional[str]:
        """Redirect the call to SCADE IDE."""
        # dynamic and undisclosed API
        return scade.command(extension_id, command_id, command)  # type: ignore reportAttributeAccessIssue

    def create_browser(
        self,
        name: str,
        icon_file: Optional[str] = None,
        keep: bool = False,
        callback: Callable[[Any, Any], None] = None,  # type: ignore
    ):
        """Redirect the call to SCADE IDE."""
        # issue with 'icon_file' default value (2025 R2)
        if icon_file:
            scade.create_browser(name, icon_file, keep, callback)  # type: ignore
        else:
            scade.create_browser(name, keep=keep, callback=callback)  # type: ignore

    def create_report(self, tab_name: str, *header: Tuple[str, int, int], check: bool = False):
        """Redirect the call to SCADE IDE."""
        scade.create_report(tab_name, *header, check)  # type: ignore

    def get_active_configuration(self, project: Project, tool_name: str) -> Optional[Configuration]:
        """Redirect the call to SCADE IDE."""
        return scade.get_active_configuration(project, tool_name)

    def get_active_project(self) -> Project:
        """Redirect the call to SCADE IDE."""
        return scade.get_active_project()  # type: ignore

    def locate(self, locate_string: str):
        """Redirect the call to SCADE IDE."""
        scade.locate(locate_string)  # type: ignore

    def locate_ex(self, tuples: Union[List[suite.Object], List[Tuple]]):
        """Redirect the call to the API."""
        suite.locate(tuples)  # type: ignore

    def open_document_view(self, file_name: str):
        """Redirect the call to SCADE IDE."""
        scade.open_document_view(file_name)  # type: ignore

    def open_html_view(
        self,
        file: Union[str, List[str]],
        use: Optional[str] = None,
        delete: bool = False,
    ):
        """Redirect the call to SCADE IDE."""
        scade.open_html_view(file, use, delete)  # type: ignore

    def open_html_in_browser(self, file_name: str):
        """Redirect the call to SCADE IDE."""
        scade.open_html_in_browser(file_name)  # type: ignore

    def open_source_code_view(self, file_name: str, line: int = 1, col: int = 1):
        """Redirect the call to SCADE IDE."""
        scade.open_source_code_view(file_name, line, col)  # type: ignore

    def output_log(
        self,
        tab_name: str,
        command: str,  # 'on' | 'off'
        path_name: str = '',
        separator: str = '',
    ):
        """Redirect the call to SCADE IDE."""
        scade.output_log(tab_name, command, path_name, separator)  # type: ignore

    def print(
        self,
        source_object: Any,
        path_name: str,
        format: str,  # 'emf' | 'text'| 'png' | 'ps'
        rotation: int = 0,
    ):
        """Redirect the call to SCADE IDE."""
        scade.print(source_object, path_name, format, rotation)

    def printer_setup(self, printer_name: str):
        """Redirect the call to SCADE IDE."""
        scade.printer_setup(printer_name)  # type: ignore

    def print_ssl(
        self,
        scade_operator: suite.Operator,
        path_name: str,
        format: str,  # 'emf' | 'png' | 'ps'
        rotation: int = 0,
    ):
        """Redirect the call to the API."""
        suite.print_ssl(scade_operator, path_name, format, rotation)

    def register_decoration(self, name: str, small_icon: str, large_icon: str):
        """Redirect the call to the API."""
        # function defined at runtime
        suite.register_decoration(name, str(small_icon), str(large_icon))  # type: ignore

    def report(self, item: Any, *columns: str):
        """Redirect the call to SCADE IDE."""
        scade.report(item, *columns)  # type: ignore

    @property
    def selection(self) -> List[Any]:
        """Redirect the call to SCADE IDE."""
        return scade.selection  # type: ignore

    @selection.setter
    def selection(self, selection: List[Any]):
        """Stub ``scade.selection``."""
        scade.selection = selection

    def set_decoration(self, object_: suite.Object, name: str):
        """Redirect the call to the API."""
        # function defined at runtime
        suite.set_decoration(object_, name)  # type: ignore

    def set_output_tab(self, tab: str):
        """Redirect the call to SCADE IDE."""
        scade.set_output_tab(tab)  # type: ignore

    def set_style(self, presentation_element: suite.PresentationElement, style_name: str):
        """Redirect the call to the API."""
        suite.set_style(presentation_element, style_name)

    def tabput(self, tab: str, text: str):
        """Redirect the call to SCADE IDE."""
        scade.tabput(tab, text)  # type: ignore

    def unset_decoration(self, object_: suite.Object):
        """Redirect the call to the API."""
        # function defined at runtime
        suite.unset_decoration(object_)  # type: ignore

    def version(
        self,
        kind: str,  # 'number' | 'folderName' | 'buildNumber' | 'endYear' | 'versionName' | 'copyright'
    ) -> str:
        """Redirect the call to SCADE IDE."""
        return scade.version(kind)  # type: ignore

    # Custom Dialogs

    def message_box(
        self, name: str, message: str, style: str = 'ok', icon: str = 'information'
    ) -> int:
        """Stub ``scade.tool.suite.gui.dialogs.message_box``."""
        return 0

    def file_open(self, filter: str = '', directory: str = '') -> str:
        """Stub ``scade.tool.suite.gui.dialogs.file_open``."""
        return ''

    def file_save(
        self, file_name: str, extension: str = '', directory: str = '', filter: str = ''
    ) -> str:
        """Stub ``scade.tool.suite.gui.dialogs.file_save``."""
        return ''

    def browse_directory(self, initial_directory: str = '') -> str:
        """Stub ``scade.tool.suite.gui.dialogs.browse_directory``."""
        return ''

    # Callback Registration

    def register_terminate_callable(self, callable):
        """Stub ``scade.tool.suite.gui.register_terminate_callable``."""
        pass

    def register_load_model_callable(self, callable):
        """Stub ``scade.tool.suite.gui.register_load_model_callable``."""
        pass

    def register_unload_model_callable(self, callable):
        """Stub ``scade.tool.suite.gui.register_unload_model_callable``."""
        pass

    # API-Related Python Commands

    def get_projects(self) -> List[Project]:
        """Redirect the call to the API."""
        return get_projects()

    def get_sessions(self) -> List[suite.Session]:
        """Redirect the call to the API."""
        return get_sessions()

    # Misc.

    def log(self, text: str):
        """Redirect the call to the local log function."""
        log(text)


def log(text: str):
    """Log the message. Temporary implementation."""
    print(text)


studio = Studio()



# -*- coding: utf-8 -*-

# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""SCADE IDE abstraction."""

from abc import ABC, abstractmethod
from typing import Any, Callable, List, Optional, Tuple, Union

from scade.model import suite
from scade.model.project.stdproject import Configuration, Project


class Ide(ABC):
    """SCADE IDE abstraction."""

    # GUI-related Python commands

    @abstractmethod
    def activate(self, object_: Any):
        """Abstract ``scade.activate``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def activate_browser(self, tab_name: str):
        """Abstract ``scade.activate_browser``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def activate_project(self, project_name: str):
        """Abstract ``scade.activate_project``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def activate_tab(self, tab: str):
        """Abstract ``scade.activate_tab``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def browser_report(
        self,
        child_object: Any,
        parent_object: Any = None,
        expanded: bool = False,
        user_data: Any = None,
        name: str = '',
        icon_file: str = '',
    ):
        """Abstract ``scade.browser_report``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def clear_tab(self, tab: str):
        """Abstract ``scade.clear_tab``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def command(
        self, extension_id: str, command_id: int, command: str = 'activate'
    ) -> Optional[str]:
        """Abstract ``scade.command (undocumented API)``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def create_browser(
        self,
        name: str,
        icon_file: Optional[str] = None,
        keep: bool = False,
        callback: Callable[[Any, Any], None] = None,
    ):
        """Abstract ``scade.create_browser``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def create_report(self, tab_name: str, *header: Tuple[str, int, int], check: bool = False):
        """Abstract ``scade.create_report``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def get_active_configuration(self, project: Project, tool_name: str) -> Optional[Configuration]:
        """Abstract ``scade.get_active_configuration``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def get_active_project(self) -> Optional[Project]:
        """Abstract ``scade.get_active_project``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def locate(self, locate_string: str):
        """Abstract ``scade.locate``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def locate_ex(self, tuples: Union[List[suite.Object], List[Tuple]]):
        """Abstract ``scade.model.suite.locate``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def open_document_view(self, file_name: str):
        """Abstract ``scade.open_document_view``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def open_html_view(
        self,
        file: Union[str, List[str]],
        use: Optional[str] = None,
        delete: bool = False,
    ):
        """Abstract ``scade.open_html_view``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def open_html_in_browser(self, file_name: str):
        """Abstract ``scade.open_html_in_browser``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def open_source_code_view(self, file_name: str, line: int = 1, col: int = 1):
        """Abstract ``scade.open_source_code_view``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def output_log(
        self,
        tab_name: str,
        command: str,  # 'on' | 'off'
        path_name: str = '',
        separator: str = '',
    ):
        """Abstract ``scade.output_log``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def print(
        self,
        source_object: Any,
        path_name: str,
        format: str,  # 'emf' | 'text'| 'png' | 'ps'
        rotation: int = 0,
    ):
        """Abstract ``scade.print``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def printer_setup(self, printer_name: str):
        """Abstract ``scade.printer_setup``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def print_ssl(
        self,
        scade_operator: suite.Operator,
        path_name: str,
        format: str,  # 'emf' | 'png' | 'ps'
        rotation: int = 0,
    ):
        """Abstract ``scade.model.suite.print_ssl``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def register_decoration(self, name: str, small_icon: str, large_icon: str):
        """Abstract ``scade.model.suite.register_decoration``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def report(self, item: Any, *columns: str):
        """Abstract ``scade.report``."""
        raise NotImplementedError('Abstract method call')

    @property
    @abstractmethod
    def selection(self) -> List[Any]:
        """Abstract ``scade.selection``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def set_decoration(self, object_: suite.Object, name: str):
        """Abstract ``scade.model.suite.set_decoration``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def set_output_tab(self, tab: str):
        """Abstract ``scade.set_output_tab``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def set_style(self, presentation_element: suite.PresentationElement, style_name: str):
        """Abstract ``scade.model.suite.set_style``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def tabput(self, tab: str, text: str):
        """Abstract ``scade.tabput``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def unset_decoration(self, object_: suite.Object):
        """Abstract ``scade.model.suite.set_decoration``."""
        raise NotImplementedError('Abstract method call')

    @property
    @abstractmethod
    def version(
        self, kind: str
    ) -> str:  # 'number' | 'folderName' | 'buildNumber' | 'endYear' | 'versionName' | 'copyright'
        """Abstract ``scade.version``."""
        raise NotImplementedError('Abstract method call')

    # Custom Dialogs

    @abstractmethod
    def message_box(
        self, name: str, message: str, style: str = 'ok', icon: str = 'information'
    ) -> int:
        """Abstract ``scade.tool.suite.gui.dialogs.message_box``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def file_open(self, filter: str = '', directory: str = '') -> str:
        """Abstract ``scade.tool.suite.gui.dialogs.file_open``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def file_save(
        self, file_name: str, extension: str = '', directory: str = '', filter: str = ''
    ) -> str:
        """Abstract ``scade.tool.suite.gui.dialogs.file_save``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def browse_directory(self, initial_directory: str = '') -> str:
        """Abstract ``scade.tool.suite.gui.dialogs.file_save``."""
        raise NotImplementedError('Abstract method call')

    # Callback Registration

    @abstractmethod
    def register_terminate_callable(self, callable):
        """Abstract ``scade.tool.suite.gui.register_terminate_callable``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def register_load_model_callable(self, callable):
        """Abstract ``scade.tool.suite.gui.register_load_model_callable``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def register_unload_model_callable(self, callable):
        """Abstract ``scade.tool.suite.gui.register_unload_model_callable``."""
        raise NotImplementedError('Abstract method call')

    # API-Related Python Commands

    @abstractmethod
    def get_projects(self) -> List[Project]:
        """Abstract ``scade.model.project.stdproject.get_roots``."""
        raise NotImplementedError('Abstract method call')

    @abstractmethod
    def get_sessions(self) -> List[suite.Session]:
        """Abstract ``scade.model.suite.get_roots``."""
        raise NotImplementedError('Abstract method call')

    # Misc.

    @abstractmethod
    def log(self, text: str):
        """Abstract ``scade.output``."""
        raise NotImplementedError('Abstract method call')



# -*- coding: utf-8 -*-

# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""``Ide`` interface implementation for custom extensions."""

# this module uses `type: ignore` pragmas for ``scade`` functions: they
# are defined dynamically when the interpreter is created: typing analysis
# is not possible

from scade.tool.suite.gui import (
    dialogs,
    # functions defined dynamically
    register_load_model_callable,  # type: ignore
    register_terminate_callable,  # type: ignore
    register_unload_model_callable,  # type: ignore
)

from ansys.scade.guitools.studio import Studio


class StudioExt(Studio):
    """Implementation for SCADE IDE."""

    # Custom Dialogs

    def message_box(
        self, name: str, message: str, style: str = 'ok', icon: str = 'information'
    ) -> int:
        """Redirect the call to the API."""
        return dialogs.message_box(name, message, style, icon)

    def file_open(self, filter: str = '', directory: str = '') -> str:
        """Redirect the call to the API."""
        return dialogs.file_open(filter, directory)

    def file_save(
        self, file_name: str, extension: str = '', directory: str = '', filter: str = ''
    ) -> str:
        """Redirect the call to the API."""
        return dialogs.file_save(file_name, extension, directory, filter)

    def browse_directory(self, initial_directory: str = '') -> str:
        """Redirect the call to the API."""
        return dialogs.browse_directory(initial_directory)

    # Callback Registration

    def register_terminate_callable(self, callable):
        """Redirect the call to the API."""
        register_terminate_callable(callable)

    def register_load_model_callable(self, callable):
        """Redirect the call to the API."""
        register_load_model_callable(callable)

    def register_unload_model_callable(self, callable):
        """Redirect the call to the API."""
        register_unload_model_callable(callable)


studio = StudioExt()



# -*- coding: utf-8 -*-

# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Provides extensions for new or existing controls."""

from enum import Enum
import os
from pathlib import Path
import re
from typing import (
    Any,
    Dict,  # noqa: F401  #used in a typing annotation
    List,
    Optional,
    Tuple,
)

from scade.tool.suite.gui.dialogs import browse_directory, file_open, file_save
from scade.tool.suite.gui.widgets import (
    Button,
    CheckBox,
    ComboBox as _ComboBox,
    EditBox,
    GroupBox,
    Label,
    ObjectComboBox as _ObjectComboBox,
    RadioButton,
    Widget,
)

import ansys.scade.guitools.csts as c


class PushButton(Button):
    """
    Defines a button control with a default size.

    Parameters
    ----------
    owner : Any
        Owner of the button.

    name : str
        Name of the button.

    x : int
        Horizontal position of the push button.

    y : int
        Vertical position of the push button.

    w : int
        Width of the push button, default csts.BUTTON_WIDTH.

    h : int
        Height of the push button, default csts.BUTTON_HEIGHT.

    kwargs : Any
        Other parameters of ``scade.tool.suite.gui.widgets.Button``.
    """

    def __init__(
        self,
        owner: Any,
        name: str,
        x: int,
        y: int,
        w: int = c.BUTTON_WIDTH,
        h: int = c.BUTTON_HEIGHT,
        **kwargs,
    ):
        """Initialize the push button with the given parameters."""
        super().__init__(owner, name, x, y, w, h, **kwargs)


class Edit(EditBox):
    """
    Defines a edit box control with a default height.

    Parameters
    ----------
    owner : Any
        Owner of the edit control.

    x : int
        Horizontal position of the edit control.

    y : int
        Vertical position of the edit control.

    w : int
        Width of the edit control.

    h : int
        Height of the edit control, default csts.EDIT_HEIGHT.

    kwargs : Any
        Other parameters of ``scade.tool.suite.gui.widgets.EditBox``.
    """

    def __init__(self, owner, x: int, y: int, w: int, h: int = c.EDIT_HEIGHT, **kwargs):
        """Initialize the edit control with the given parameters."""
        super().__init__(owner, x, y, w, h, **kwargs)
        self.owner = owner

    def on_layout(self):
        """Declare the constraints with respect to the owner."""
        self.set_constraint(Widget.RIGHT, self.owner, Widget.RIGHT, -c.RIGHT_MARGIN)


class StaticEdit(Edit):
    """
    Defines a bundle made of a static and an edit control.

    Parameters
    ----------
    owner : Any
        Owner of the control.

    text : str
        Text of the static control.

    wl : int
        Width of the static control.

    x : int
        Horizontal position of the control.

    y : int
        Vertical position of the control.

    w : int
        Width of the control.

    h : int
        Height of the control, default csts.EDIT_HEIGHT.

    kwargs : Any
        Other parameters of ``scade.tool.suite.gui.widgets.EditBox``.
    """

    def __init__(
        self,
        owner,
        text: str,
        wl: int,
        x: int,
        y: int,
        w: int,
        h: int = c.EDIT_HEIGHT,
        **kwargs,
    ):
        """Initialize the static edit control with the given parameters."""
        self.label = Label(owner, text, x, y + 4, wl, h - 4)
        super().__init__(owner, x + wl, y, w - wl, h, **kwargs)
        self.owner = owner

    def set_visible(self, show: bool):
        """Show or hide the control."""
        super().set_visible(show)
        self.label.set_visible(show)


class FSM(Enum):
    """Mode for the file selector."""

    OPEN, SAVE, DIR = range(3)


class FileSelector(StaticEdit):
    """
    Defines a bundle made of a static, an edit, and a button control.

    Parameters
    ----------
    owner : Any
        Owner of the control.

    text : str
        Text of the static control.

    extension : str
        Default extension of the files.
        This option is ignored when ``mode`` is ``FSM.DIR``.

    directory : str
        Initial directory of the file selector dialog box, or the current directory when empty.

    filter : str
        Description of the format of the visible files in the file selector dialog box.
        This option is ignored when ``mode`` is ``FSM.DIR``.

    mode : FSM
        Mode of the file seclector dialog box, either ``FSM.LOAD``, ``FSM.SAVE``,
        or ``FSM.DIR`` for browsing directories.

    wl : int
        Width of the static control.

    x : int
        Horizontal position of the control.

    y : int
        Vertical position of the control.

    w : int
        Width of the control.

    h : int
        Height of the control, default csts.EDIT_HEIGHT.

    reference : str
        Reference directory to resolve or compute a relative path when not empty.

    kwargs : Any
        Other parameters of ``scade.tool.suite.gui.widgets.EditBox``.
    """

    # separator between the edit and the button controls
    _SEPARATOR = 5

    def __init__(
        self,
        owner,
        text: str,
        extension: str,
        directory: str,
        filter: str,
        mode: FSM,
        wl: int,
        x: int,
        y: int,
        w: int,
        h: int = c.EDIT_HEIGHT,
        reference: str = '',
        **kwargs,
    ):
        """Initialize the file selector with the given parameters."""
        super().__init__(owner, text, wl, x, y, w - c.DOTS_WIDTH - self._SEPARATOR, h, **kwargs)
        x_dots = x + w - c.DOTS_WIDTH
        # so that borders are aligned
        y_dots = y - 1
        self.btn_dots = Button(
            owner,
            '...',
            x_dots,
            y_dots,
            c.DOTS_WIDTH,
            c.DOTS_HEIGHT,
            on_click=self.on_click,
        )
        self.owner = owner
        self.extension = extension
        self.directory = directory
        self.filter = filter
        self.mode = mode
        self.reference = reference

    def on_click(self, button: Button):
        """Call the Windows standard open or save selection commands."""

        def expand_vars(name: str) -> str:
            """Expand environment variables in the given string."""
            # rename $(...) by ${...}
            name = re.sub(r'\$\(([^\)/\\\$]*)\)', r'${\1}', name)
            # resolve the environment variables, if any
            return os.path.expandvars(name)

        # wrong signature for Edit.get_name()
        name: str = self.get_name()  # type: ignore

        name = expand_vars(name)
        directory = expand_vars(self.directory)
        reference = expand_vars(self.reference)
        if not directory and reference:
            directory = reference
        # ensure Windows' path syntax
        directory = str(Path(directory))
        if self.mode == FSM.SAVE:
            path = file_save(name, self.extension, directory, self.filter)
        elif self.mode == FSM.OPEN:
            path = file_open(self.filter, directory)
        else:
            # assert self.mode == FSM.DIR
            path = browse_directory(directory)
        if path:
            if reference:
                try:
                    path = os.path.relpath(path, reference)
                except ValueError:
                    pass
            self.set_name(path)

    def on_layout(self):
        """Declare the constraints with respect to the owner."""
        self.btn_dots.set_constraint(Widget.RIGHT, self.owner, Widget.RIGHT, -c.RIGHT_MARGIN)
        self.btn_dots.set_constraint(
            Widget.LEFT, self.owner, Widget.RIGHT, -c.RIGHT_MARGIN - c.DOTS_WIDTH
        )
        self.set_constraint(Widget.RIGHT, self.btn_dots, Widget.LEFT, -self._SEPARATOR)

    def set_visible(self, show: bool):
        """Show or hide the control."""
        super().set_visible(show)
        self.btn_dots.set_visible(show)


class CheckButton(CheckBox):
    """
    Defines a check box control with a default height.

    Parameters
    ----------
    owner : Any
        Owner of check button.

    text : str
        Text of check button.

    x : int
        Horizontal position of the check button.

    y : int
        Vertical position of the check button.

    w : int
        Width of the check button.

    h : int
        Height of the check button, default csts.CHECK_BUTTON_HEIGHT.

    kwargs : Any
        Other parameters of ``scade.tool.suite.gui.widgets.CheckBox``.
    """

    def __init__(
        self,
        owner,
        text: str,
        x: int,
        y: int,
        w: int,
        h: int = c.CHECK_BUTTON_HEIGHT,
        **kwargs,
    ):
        """Initialize the check button with the given parameters."""
        super().__init__(owner, text, x, y, w, h, **kwargs)
        self.owner = owner


class ComboBox(_ComboBox):
    """
    Defines a combo box control with a default height.

    Parameters
    ----------
    owner : Any
        Owner of the combo box control.

    x : int
        Horizontal position of the combo box control.

    y : int
        Vertical position of the combo box control.

    w : int
        Width of the combo box control.

    h : int
        Height of the combo box control, default csts.COMBO_BOX_HEIGHT.

    kwargs : Any
        Other parameters of ``scade.tool.suite.gui.widgets.ComboBox``.
    """

    def __init__(self, owner, x: int, y: int, w: int, h: int = c.COMBO_BOX_HEIGHT, **kwargs):
        """Initialize the combo box with the given parameters."""
        super().__init__(owner, [], x, y, w, h, **kwargs)
        self.owner = owner

    def on_layout(self):
        """Declare the constraints with respect to the owner."""
        self.set_constraint(Widget.RIGHT, self.owner, Widget.RIGHT, -c.RIGHT_MARGIN)


class ObjectComboBox(_ObjectComboBox):
    """
    Defines an object combo box control with a default height and extensions for serialization.

    This class provides an optional map to access an item with a string. This allows
    to persist the current selection in a file, or restore it from a value.

    Parameters
    ----------
    owner : Any
        Owner of the combo box control.

    x : int
        Horizontal position of the combo box control.

    y : int
        Vertical position of the combo box control.

    w : int
        Width of the combo box control.

    h : int
        Height of the combo box control, default csts.COMBO_BOX_HEIGHT.

    kwargs : Any
        Other parameters of ``scade.tool.suite.gui.widgets.ComboBox``.
    """

    def __init__(self, owner, x: int, y: int, w: int, h: int = c.COMBO_BOX_HEIGHT, **kwargs):
        """Initialize the object combo box with the given parameters."""
        super().__init__(owner, [], x, y, w, h, **kwargs)
        self.owner = owner
        self.items = []  # type: List[Any]
        self.names = []  # type: List[str]

    def on_layout(self):
        """Declare the constraints with respect to the owner."""
        self.set_constraint(Widget.RIGHT, self.owner, Widget.RIGHT, -c.RIGHT_MARGIN)

    def set_items(self, items: List[Any], names: Optional[List[str]] = None):
        """
        Set the combo box items, with an optional mapping.

        This mapping is used to serialize the selected item.
        """
        super().set_items(items)
        self.items = items
        self.names = names if names else []

    def get_selected_name(self) -> str:
        """Return the name of the selected item."""
        item = self.get_selection()
        try:
            name = self.names[self.items.index(item)]
        except ValueError:
            name = ''
        return name

    def select_name(self, name: str):
        """Select the item corresponding to name."""
        try:
            item = self.items[self.names.index(name)]
        except ValueError:
            item = None
        self.set_selection(item)


class StaticComboBox(ComboBox):
    """
    Defines a bundle made of a static and a combo box control.

    Parameters
    ----------
    owner : Any
        Owner of the control.

    text : str
        Text of the static control.

    wl : int
        Width of the static control.

    x : int
        Horizontal position of the control.

    y : int
        Vertical position of the control.

    w : int
        Width of the control.

    h : int
        Height of the control, default csts.COMBO_BOX_HEIGHT.

    kwargs : Any
        Other parameters of ``scade.tool.suite.gui.widgets.ComboBox``.
    """

    def __init__(
        self,
        owner,
        text: str,
        wl: int,
        x: int,
        y: int,
        w: int,
        h: int = c.COMBO_BOX_HEIGHT,
        **kwargs,
    ):
        """Initialize the static combo box with the given parameters."""
        self.label = Label(owner, text, x, y + 4, wl, c.STATIC_HEIGHT)
        super().__init__(owner, x + wl, y, w - wl, h, **kwargs)
        self.owner = owner

    def set_visible(self, show: bool):
        """Show or hide the control."""
        super().set_visible(show)
        self.label.set_visible(show)


class StaticObjectComboBox(ObjectComboBox):
    """
    Defines a bundle made of a static and an object combo box control.

    Parameters
    ----------
    owner : Any
        Owner of the control.

    text : str
        Text of the static control.

    wl : int
        Width of the static control.

    x : int
        Horizontal position of the control.

    y : int
        Vertical position of the control.

    w : int
        Width of the control.

    h : int
        Height of the control, default csts.COMBO_BOX_HEIGHT.

    kwargs : Any
        Other parameters of ``scade.tool.suite.gui.widgets.ObjectComboBox``.
    """

    def __init__(
        self,
        owner,
        text: str,
        wl: int,
        x: int,
        y: int,
        w: int,
        h: int = c.COMBO_BOX_HEIGHT,
        style: Optional[List[str]] = None,
        **kwargs,
    ):
        """Initialize the static object combo box with the given parameters."""
        if not style:
            style = []
        if 'dropdownlist' not in style:
            style.append('dropdownlist')
        self.label = Label(owner, text, x, y + 4, wl, c.STATIC_HEIGHT)
        super().__init__(owner, x + wl, y, w - wl, h, style=style, **kwargs)
        self.owner = owner

    def set_visible(self, show: bool):
        """Show or hide the control."""
        super().set_visible(show)
        self.label.set_visible(show)


class RadioBox(GroupBox):
    """
    Defines a bundle made of a group and a set of radio button controls.

    The group is hidden when text is empty.

    Parameters
    ----------
    owner : Any
        Owner of the control.

    buttons : list[tuple[str, str]]
        Descriptions of the buttons: value and text associated to the buttons.

    x : int
        Horizontal position of the control.

    y : int
        Vertical position of the control.

    w : int
        Width of the control.

    h : int
        Height of the control, default 0.

    text : str
        Text of the group control, default empty.

    kwargs : Any
        Other parameters of ``scade.tool.suite.gui.widgets.GroupBox``.
    """

    def __init__(
        self,
        owner,
        buttons: List[Tuple[str, str]],
        x: int,
        y: int,
        w: int,
        text: str = '',
        h: int = c.GROUP_RADIO_BOX_HEIGHT,
    ):
        """Initialize the radio box with the given parameters."""
        if text:
            # group visible
            offset_x = c.LEFT_MARGIN
            offset_y = c.STATIC_HEIGHT
            # width for buttons
            width = w - c.LEFT_MARGIN - c.RIGHT_MARGIN
        else:
            # group not visible
            offset_x = 0
            offset_y = 0
            width = w
            h = 0

        n = len(buttons)
        wb = int(width / n) if n > 0 else width
        self.buttons = {}  # type: Dict[str, RadioButton]
        self.owner = owner
        self.text = text
        # the group is used to set relative constraints
        super().__init__(owner, text, x, y, w, h)
        x = x + offset_x
        y = y + offset_y
        for value, text in buttons:
            button = RadioButton(owner, text, x, y, wb, c.RADIO_BUTTON_HEIGHT)
            self.buttons[value] = button
            x += wb
        Widget.group(self.buttons.values())

    def on_layout(self):
        """Declare the constraints with respect to the owner."""
        self.set_constraint(Widget.RIGHT, self.owner, Widget.RIGHT, -c.RIGHT_MARGIN)
        prev = None
        count = len(self.buttons)
        margin = c.LEFT_MARGIN + c.RIGHT_MARGIN if self.text else 0
        for button in self.buttons.values():
            if prev:
                button.set_constraint(Widget.LEFT, prev, Widget.RIGHT, 0)
            button.set_constraint(
                # wrong signature for set_constraint(), mul parameter must be float
                Widget.WIDTH,
                self,
                Widget.WIDTH,
                -int(margin / count),
                1.0 / count,  # type: ignore
            )
            # button.set_constraint(Widget.WIDTH, self, Widget.WIDTH, 0, 1. / count)
            prev = button

    def set_visible(self, show: bool):
        """Show or hide the control."""
        self.set_visible(show)
        for button in self.buttons.values():
            button.set_visible(show)

    def get_value(self) -> str:
        """Return the value of the selected button, or ``""`` when none is selected."""
        for value, button in self.buttons.items():
            if button.get_check():
                return value
        else:
            return ''

    def set_value(self, value: str):
        """
        Select the button corresponding to the input value.

        * No button is selected when ``value`` is ``""``.
        * The first button is selected if ``value`` does not correspond to any button.

        Parameters
        ----------
        value : str
            Input value corresponding to a button.
        """
        if value == '':
            # uncheck all buttons
            for button in self.buttons.values():
                button.set_check(False)
        else:
            if value not in self.buttons:
                value = list(self.buttons.keys())[0]
            for v, button in self.buttons.items():
                button.set_check(v == value)


class GroupRadioBox(RadioBox):
    """
    Defines a bundle made of a group and a set of radio button controls.

    Parameters
    ----------
    owner : Any
        Owner of the control.

    text : str
        Text of the group control.

    buttons : list[tuple[str, str]]
        Descriptions of the buttons: value and text associated to the buttons.

    x : int
        Horizontal position of the control.

    y : int
        Vertical position of the control.

    w : int
        Width of the control.

    h : int
        Height of the control, default csts.RADIO_BOX_HEIGHT.

    kwargs : Any
        Other parameters of ``scade.tool.suite.gui.widgets.GroupBox``.
    """

    def __init__(
        self,
        owner,
        text: str,
        buttons: List[Tuple[str, str]],
        x: int,
        y: int,
        w: int,
        h: int = c.GROUP_RADIO_BOX_HEIGHT,
    ):
        """Initialize the group radio box with the given parameters."""
        super().__init__(owner, buttons, x, y, w, text=text, h=h)


class StaticRadioBox(RadioBox):
    """
    Defines a bundle made of a static and a group of radio button controls.

    The group is not visible and is used to set relative constraints.

    Parameters
    ----------
    owner : Any
        Owner of the control.

    text : str
        Text of the static control.

    wl : int
        Width of the static control.

    buttons : list[tuple[str, str]]
        Descriptions of the buttons: value and text associated to the buttons.

    x : int
        Horizontal position of the control.

    y : int
        Vertical position of the control.

    w : int
        Width of the control.

    kwargs : Any
        Other parameters of ``scade.tool.suite.gui.widgets.GroupBox``.
    """

    def __init__(
        self,
        owner,
        text: str,
        wl: int,
        buttons: List[Tuple[str, str]],
        x: int,
        y: int,
        w: int,
    ):
        """Initialize the static radio box with the given parameters."""
        self.label = Label(owner, text, x=x, y=y + 4, w=wl, h=c.STATIC_HEIGHT)
        super().__init__(owner, buttons, x + wl, y, w - wl)

    def set_visible(self, show: bool):
        """Show or hide the control."""
        self.label.set_visible(show)
        super().set_visible(show)



# -*- coding: utf-8 -*-

# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Interfaces for GUI Host pages and data persistence."""

from abc import ABC, abstractmethod
from typing import Any, List

from scade.model.project.stdproject import Configuration, Project
from scade.tool.suite.gui.properties import Page as PropertyPage


class ISettingsDataExchange(ABC):
    """Interface for exchanging data between a settings page and the model."""

    @abstractmethod
    def model_to_page(self, project: Project, configuration: Configuration):
        """
        Update the page with the properties read from the project for a given configuration.

        Parameters
        ----------
        project : Project
            Input Project.

        configuration : Configuration
            Input configuration.
        """
        raise NotImplementedError

    @abstractmethod
    def page_to_model(self, project: Project, configuration: Configuration):
        """
        Update the project with the properties read from the page for a given configuration.

        Parameters
        ----------
        project : Project
            Input Project.

        configuration : Configuration
            Input configuration.
        """
        raise NotImplementedError


class IPropertiesDataExchange(ABC):
    """Interface for exchanging data between a property page and the model."""

    @abstractmethod
    def model_to_page(self, model: Any):
        """
        Update the page with the properties read from the model.

        Parameters
        ----------
        model : Any
            Selected model element.
        """
        raise NotImplementedError

    @abstractmethod
    def page_to_model(self, model: Any):
        """
        Update the model with the properties read from the page.

        Parameters
        ----------
        model : Any
            Selected model element.
        """
        raise NotImplementedError


class IGuiHostClient(ABC):
    """Defines the interface for hosted pages."""

    @abstractmethod
    def is_available(self, models: List[Any]) -> bool:
        """
        Return whether the page is available for the current selection.

        Parameters
        ----------
        models : List[Any]
            List of selected objects in the IDE.
        """
        raise NotImplementedError

    @abstractmethod
    def set_models(self, models: List[Any]):
        """
        Declare the models the page should consider.

        Parameters
        ----------
        models : List[Any]
            List of selected objects in the IDE.
        """
        raise NotImplementedError

    @abstractmethod
    def on_build(self, page: PropertyPage, y: int) -> int:
        """
        Build the controls.

        Parameters
        ----------
        page : PropertyPage
            Owning property page, to create controls.

        y : int
            Start vertical position.
        """
        raise NotImplementedError

    @abstractmethod
    def show(self, show: bool):
        """
        Show or hide the page.

        This consists in showing or hiding the contained controls.

        Parameters
        ----------
        show : bool
            Whether the page should be shown or hidden.
        """
        raise NotImplementedError

    @abstractmethod
    def on_layout(self):
        """Declare the contained control's constraints."""
        raise NotImplementedError

    @abstractmethod
    def on_display(self):
        """Update the page with the properties read from the models."""
        raise NotImplementedError

    @abstractmethod
    def on_validate(self):
        """Update the models with the properties read from the page."""
        raise NotImplementedError

    @abstractmethod
    def on_close(self):
        """Perform any cleaning before the page is closed."""
        raise NotImplementedError



# -*- coding: utf-8 -*-

# Copyright (C) 2024 - 2025 ANSYS, Inc. and/or its affiliates.
# SPDX-License-Identifier: MIT
#
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

"""Stubs for unit tests."""

import json
from pathlib import Path
from typing import Any, Callable, Dict, List, Optional, Tuple, Union

from scade.model.project.stdproject import Configuration, Project
import scade.model.suite as suite

from ansys.scade.apitools.info.install import get_scade_properties
from ansys.scade.guitools.ide import Ide


class StubIde(Ide):
    """
    SCADE IDE instantiation for unit tests.

    This class stubs most popular functions by storing inputs so that they
    can be compared with expected data.

    It can be sub-classed for specific use-cases.
    """

    def __init__(self):
        self.project: Optional[Project] = None
        self.session: Optional[suite.Session] = None
        self._selection = []
        self.browser = None
        self.browser_items: Dict[Any, Any] = {}
        # report
        # * tab: name of active report
        # * items: list of tuples, first element is the header
        self.report_tab: str = ''
        self.report_items: List[list] = []
        # decorations
        self.decorations: list[str] = []
        self.decorated_items: Dict[suite.Object, str] = {}

    # GUI-related Python commands

    def activate(self, object_: Any):
        """Ignore the call."""
        pass

    def activate_browser(self, tab_name: str):
        """Ignore the call."""
        pass

    def activate_project(self, project_name: str):
        """Ignore the call."""
        pass

    def activate_tab(self, tab: str):
        """Ignore the call."""
        pass

    def browser_report(
        self,
        child_object: Any,
        parent_object: Any = None,
        expanded: bool = False,
        user_data: Any = None,
        name: str = '',
        icon_file: str = '',
    ):
        """Stub ``scade.browser_report``."""
        assert self.browser_items is not None
        if isinstance(child_object, str):
            child = child_object
        else:
            # try some common attributes
            for att in ['name', 'pathname']:
                name = getattr(child_object, att, '')
                if name:
                    break
            else:
                name = 'anonymous'
            child = f'<{type(child_object).__name__}> {name}'
        parent = self.browser_items[parent_object]
        entry = {
            'name': child,
            'icon_file': Path(icon_file).name if icon_file else '',
            'expanded': expanded,
            'user_date': user_data,
            'children': [],
        }
        parent['children'].append(entry)
        self.browser_items[child_object] = entry

    def clear_tab(self, tab: str):
        """Ignore the call."""
        pass

    def command(
        self, extension_id: str, command_id: int, command: str = 'activate'
    ) -> Optional[str]:
        """Ignore the call."""
        return None

    def create_browser(
        self,
        name: str,
        icon_file: Optional[str] = None,
        keep: bool = False,
        callback: Callable[[Any, Any], None] = None,  # type: ignore
    ):
        """Stub ``scade.create_browser``."""
        self.browser = {
            'name': name,
            'icon': Path(icon_file).name if icon_file else '',
            'children': [],
        }
        self.browser_items = {None: self.browser}

    def create_report(self, tab_name: str, *header: Tuple[str, int, int], check: bool = False):
        """Stub ``scade.create_report``."""
        self.report_tab = tab_name
        self.report_items = [[_[0] for _ in header]]

    def get_active_configuration(self, project: Project, tool_name: str) -> Optional[Configuration]:
        """Stub ``scade.get_active_configuration``."""
        return None

    def get_active_project(self) -> Project:
        """Stub ``scade.get_active_project``."""
        assert self.project is not None
        return self.project

    def locate(self, locate_string: str):
        """Ignore the call."""
        pass

    def locate_ex(self, tuples: Union[List[suite.Object], List[Tuple]]):
        """Ignore the call."""
        pass

    def open_document_view(self, file_name: str):
        """Ignore the call."""
        pass

    def open_html_view(
        self,
        file: Union[str, List[str]],
        use: Optional[str] = None,
        delete: bool = False,
    ):
        """Ignore the call."""
        pass

    def open_html_in_browser(self, file_name: str):
        """Ignore the call."""
        pass

    def open_source_code_view(self, file_name: str, line: int = 1, col: int = 1):
        """Ignore the call."""
        pass

    def output_log(self, tab_name: str, command: str, path_name: str = '', separator: str = ''):
        """Ignore the call."""
        pass

    def print(
        self,
        source_object: Any,
        path_name: str,
        format: str,  # 'emf' | 'text'| 'png' | 'ps'
        rotation: int = 0,
    ):
        """Ignore the call."""
        pass

    def printer_setup(self, printer_name: str):
        """Ignore the call."""
        pass

    def print_ssl(
        self,
        scade_operator: suite.Operator,
        path_name: str,
        format: str,  # 'emf' | 'png' | 'ps'
        rotation: int = 0,
    ):
        """Ignore the call."""
        pass

    def register_decoration(self, name: str, small_icon: str, large_icon: str):
        """Stub suite.register_decoration."""
        self.decorations.append(name)

    def report(self, item: Any, *columns: str):
        """Stub ``scade.report``."""
        self.report_items.append([item] + list(columns))

    @property
    def selection(self) -> List[Any]:
        """Stub return scade.selection."""
        return self._selection

    @selection.setter
    def selection(self, selection: List[Any]):
        """Stub ``scade.selection``."""
        self._selection = selection

    def set_decoration(self, object_: suite.Object, name: str):
        """Stub suite.set_decoration."""
        assert name in self.decorations
        self.decorated_items[object_] = name

    def set_output_tab(self, tab: str):
        """Ignore the call."""
        pass

    def set_style(self, presentation_element: suite.PresentationElement, style_name: str):
        """Ignore the call."""
        pass

    def tabput(self, tab: str, text: str):
        """Ignore the call."""
        pass

    def unset_decoration(self, object_: suite.Object):
        """Ignore the call."""
        self.decorated_items.pop(object_, None)

    def version(
        self, kind: str
    ) -> str:  # 'number' | 'folderName' | 'buildNumber' | 'endYear' | 'versionName' | 'copyright'
        """Stub ``scade.version``."""
        mapping = {
            'number': 'SCADE_STUDIO_NUMBER',
            'folderName': 'INSTALL_FOLDER',
            'buildNumber': 'BUILD_NUMBER',
            'endYear': 'COPYRIGHT_END_DATE',
            'versionName': 'RELEASE',
            'copyright': 'RELEASE_COPYRIGHT',
        }
        props = get_scade_properties()
        return props.get(mapping[kind], '<unknown>')

    # Custom Dialogs

    def message_box(
        self, name: str, message: str, style: str = 'ok', icon: str = 'information'
    ) -> int:
        """Ignore the call."""
        return 0

    def file_open(self, filter: str = '', directory: str = '') -> str:
        """Ignore the call."""
        return ''

    def file_save(
        self, file_name: str, extension: str = '', directory: str = '', filter: str = ''
    ) -> str:
        """Ignore the call."""
        return ''

    def browse_directory(self, initial_directory: str = '') -> str:
        """Ignore the call."""
        return ''

    # Callback Registration

    def register_terminate_callable(self, callable):
        """Ignore the call."""
        pass

    def register_load_model_callable(self, callable):
        """Ignore the call."""
        pass

    def register_unload_model_callable(self, callable):
        """Ignore the call."""
        pass

    # API-Related Python Commands

    def get_projects(self) -> List[Project]:
        """Stub ``scade.model.project.stdproject.get_roots``."""
        return [self.get_active_project()]

    def get_sessions(self) -> List[suite.Session]:
        """Stub ``scade.model.suite.get_roots``."""
        assert self.session is not None
        return [self.session]

    # Misc.

    def log(self, text: str):
        """Redirect the call to the local log function."""
        log(text)

    def save_browser(self, path: Path):
        """Store the current browser as a json file."""
        with path.open('w') as f:
            json.dump(self.browser, f, indent='   ', sort_keys=True)


def log(text: str):
    """Log the message. Temporary implementation."""
    print(text)


studio = StubIde()
